declare module "utils/misc/isElementType" {
    type tag = keyof HTMLElementTagNameMap;
    export function isElementType<T extends tag, P extends {
        [k: string]: unknown;
    } | undefined = undefined>(element: Element, tag: T | T[], props?: P): element is P extends undefined ? HTMLElementTagNameMap[T] : HTMLElementTagNameMap[T] & P;
}
declare module "utils/click/isClickableInput" {
    export function isClickableInput(element: Element): boolean;
}
declare module "utils/dataTransfer/Blob" {
    export function readBlobText(blob: Blob): Promise<string>;
}
declare module "utils/dataTransfer/FileList" {
    export function createFileList(files: File[]): FileList;
}
declare module "utils/dataTransfer/DataTransfer" {
    export function createDataTransfer(files?: File[]): DataTransfer;
    export function getBlobFromDataTransferItem(item: DataTransferItem): Blob;
}
declare module "utils/dataTransfer/Clipboard" {
    export function createClipboardItem(...blobs: Array<Blob | string>): ClipboardItem;
    export function attachClipboardStubToView(window: Window & typeof globalThis): {
        resetClipboardStub: () => void;
        detachClipboardStub: () => void;
    };
    export function resetClipboardStubOnView(window: Window & typeof globalThis): void;
    export function detachClipboardStubFromView(window: Window & typeof globalThis): void;
    export function readDataTransferFromClipboard(document: Document): Promise<DataTransfer>;
    export function writeDataTransferToClipboard(document: Document, clipboardData: DataTransfer): Promise<void>;
}
declare module "utils/edit/buildTimeValue" {
    export function buildTimeValue(value: string): string;
}
declare module "document/interceptor" {
    type anyFunc = (...a: any[]) => any;
    type Params<Prop> = Prop extends anyFunc ? Parameters<Prop> : [Prop];
    type ImplReturn<Prop> = Prop extends anyFunc ? Parameters<Prop> : Prop;
    export function prepareInterceptor<ElementType extends Node, PropName extends keyof ElementType>(element: ElementType, propName: PropName, interceptorImpl: (this: ElementType, ...args: Params<ElementType[PropName]>) => {
        /**
         * React tracks the changes on element properties.
         * This workaround tries to alter the DOM element without React noticing,
         * so that it later picks up the change.
         *
         * @see https://github.com/facebook/react/blob/148f8e497c7d37a3c7ab99f01dec2692427272b1/packages/react-dom/src/client/inputValueTracking.js#L51-L104
         */
        applyNative?: boolean;
        realArgs?: ImplReturn<ElementType[PropName]>;
    }): void;
}
declare module "document/selection" {
    const UISelection: unique symbol;
    export interface UISelectionRange {
        startOffset: number;
        endOffset: number;
    }
    export interface UISelection {
        anchorOffset: number;
        focusOffset: number;
    }
    global {
        interface Element {
            [UISelection]?: UISelection;
        }
    }
    export function prepareSelectionInterceptor(element: HTMLInputElement | HTMLTextAreaElement): void;
    export function setUISelection(element: HTMLInputElement | HTMLTextAreaElement, { focusOffset: focusOffsetParam, anchorOffset: anchorOffsetParam, }: {
        anchorOffset?: number;
        focusOffset: number;
    }, mode?: 'replace' | 'modify'): void;
    export function getUISelection(element: HTMLInputElement | HTMLTextAreaElement): {
        startOffset: number;
        endOffset: number;
        anchorOffset: number;
        focusOffset: number;
    };
    export function clearUISelection(element: HTMLInputElement | HTMLTextAreaElement): void;
}
declare module "document/value" {
    const UIValue: unique symbol;
    const InitialValue: unique symbol;
    const TrackChanges: unique symbol;
    global {
        interface Element {
            [UIValue]?: string;
            [InitialValue]?: string;
            [TrackChanges]?: string[];
        }
    }
    export function prepareValueInterceptor(element: HTMLInputElement): void;
    export function setUIValue(element: HTMLInputElement | HTMLTextAreaElement, value: string): void;
    export function getUIValue(element: HTMLInputElement | HTMLTextAreaElement): string;
    export function setInitialValue(element: HTMLInputElement | HTMLTextAreaElement): void;
    export function getInitialValue(element: HTMLInputElement | HTMLTextAreaElement): string | undefined;
    export function startTrackValue(element: HTMLInputElement | HTMLTextAreaElement): void;
    export function endTrackValue(element: HTMLInputElement | HTMLTextAreaElement): string[] | undefined;
}
declare module "document/index" {
    const isPrepared: unique symbol;
    global {
        interface Node {
            [isPrepared]?: typeof isPrepared;
        }
    }
    export function prepareDocument(document: Document): void;
    export { getUIValue, setUIValue, startTrackValue, endTrackValue } from "document/value";
    export { getUISelection, setUISelection } from "document/selection";
    export type { UISelectionRange } from "document/selection";
}
declare module "utils/edit/isContentEditable" {
    export function isContentEditable(element: Element): element is HTMLElement & {
        contenteditable: 'true';
    };
    /**
     * If a node is a contenteditable or inside one, return that element.
     */
    export function getContentEditable(node: Node): Element | null;
}
declare module "utils/edit/isEditable" {
    import { isContentEditable } from "utils/edit/isContentEditable";
    type GuardedType<T> = T extends (x: any) => x is infer R ? R : never;
    export function isEditable(element: Element): element is GuardedType<typeof isContentEditable> | GuardedType<typeof isEditableInput> | (HTMLTextAreaElement & {
        readOnly: false;
    });
    export enum editableInputTypes {
        'text' = "text",
        'date' = "date",
        'datetime-local' = "datetime-local",
        'email' = "email",
        'month' = "month",
        'number' = "number",
        'password' = "password",
        'search' = "search",
        'tel' = "tel",
        'time' = "time",
        'url' = "url",
        'week' = "week"
    }
    export type EditableInputType = keyof typeof editableInputTypes;
    export function isEditableInput(element: Element): element is HTMLInputElement & {
        readOnly: false;
        type: editableInputTypes;
    };
}
declare module "utils/edit/isValidDateValue" {
    export function isValidDateValue(element: HTMLInputElement & {
        type: 'date';
    }, value: string): boolean;
}
declare module "utils/edit/isValidInputTimeValue" {
    export function isValidInputTimeValue(element: HTMLInputElement & {
        type: 'time';
    }, timeValue: string): boolean;
}
declare module "utils/edit/calculateNewValue" {
    import { EditableInputType } from "utils/edit/isEditable";
    /**
     * Calculate a new text value.
     */
    export function calculateNewValue(inputData: string, node: (HTMLInputElement & {
        type: EditableInputType;
    }) | HTMLTextAreaElement, { startOffset, endOffset, }: {
        startOffset: number;
        endOffset: number;
    }, inputType?: string): {
        oldValue: string;
        newValue: string;
        newOffset: number;
    };
}
declare module "utils/focus/selection" {
    import { UISelectionRange } from "document/index";
    import { editableInputTypes } from "utils/edit/isEditable";
    /**
     * Backward-compatible selection.
     *
     * Handles input elements and contenteditable if it only contains a single text node.
     */
    export function setSelectionRange(element: Element, anchorOffset: number, focusOffset: number): void;
    /**
     * Determine if the element has its own selection implementation
     * and does not interact with the Document Selection API.
     */
    export function hasOwnSelection(node: Node): node is HTMLTextAreaElement | (HTMLInputElement & {
        type: editableInputTypes;
    });
    /**
     * Reset the Document Selection when moving focus into an element
     * with own selection implementation.
     */
    export function updateSelectionOnFocus(element: Element): void;
    /**
     * Get the range that would be overwritten by input.
     */
    export function getInputRange(focusNode: Node): UISelectionRange | Range | undefined;
    /**
     * Extend/shrink the selection like with Shift+Arrows or Shift+Mouse
     */
    export function modifySelection({ focusNode, focusOffset, }: {
        focusNode: Node;
        /** DOM Offset */
        focusOffset: number;
    }): void;
    /**
     * Set the selection
     */
    export function setSelection({ focusNode, focusOffset, anchorNode, anchorOffset, }: {
        anchorNode?: Node;
        /** DOM offset */
        anchorOffset?: number;
        focusNode: Node;
        focusOffset: number;
    }): void;
}
declare module "utils/edit/editInputElement" {
    import { fireEvent } from '@testing-library/dom';
    /**
     * Change the value of an element as if it was changed as a result of a user input.
     *
     * Fires the input event.
     */
    export function editInputElement(element: HTMLInputElement | HTMLTextAreaElement, { newValue, newSelection, eventOverrides, }: {
        newValue: string;
        newSelection: {
            node: Node;
            offset: number;
        };
        eventOverrides: Partial<Parameters<typeof fireEvent>[1]> & {
            [k: string]: unknown;
        };
    }): void;
}
declare module "utils/edit/getValue" {
    export function getValue<T extends Element | null>(element: T): T extends HTMLInputElement | HTMLTextAreaElement ? string : string | null;
}
declare module "utils/edit/maxLength" {
    export function getSpaceUntilMaxLength(element: Element): number | undefined;
}
declare module "utils/edit/prepareInput" {
    export function prepareInput(data: string, element: Element, inputType?: string): {
        commit: () => void;
        getNewValue?: undefined;
    } | {
        getNewValue: () => string;
        commit: () => void;
    } | undefined;
}
declare module "utils/edit/setFiles" {
    const fakeFiles: unique symbol;
    global {
        interface HTMLInputElement {
            [fakeFiles]?: {
                restore: () => void;
            };
        }
    }
    export function setFiles(el: HTMLInputElement & {
        type: 'file';
    }, files: FileList): void;
}
declare module "utils/misc/eventWrapper" {
    export function eventWrapper<T>(cb: () => T): T | undefined;
}
declare module "utils/misc/isDisabled" {
    export function isDisabled(element: Element | null): boolean;
}
declare module "utils/focus/getActiveElement" {
    export function getActiveElement(document: Document | ShadowRoot): Element | null;
}
declare module "utils/focus/selector" {
    export const FOCUSABLE_SELECTOR: string;
}
declare module "utils/focus/isFocusable" {
    export function isFocusable(element: Element): element is HTMLElement;
}
declare module "utils/focus/blur" {
    function blur(element: Element): void;
    export { blur };
}
declare module "utils/focus/copySelection" {
    export function copySelection(target: Element): DataTransfer;
}
declare module "utils/focus/focus" {
    function focus(element: Element): void;
    export { focus };
}
declare module "utils/misc/isVisible" {
    export function isVisible(element: Element): boolean;
}
declare module "utils/focus/getTabDestination" {
    export function getTabDestination(activeElement: Element, shift: boolean): HTMLElement;
}
declare module "utils/focus/selectAll" {
    /**
     * Expand a selection like the browser does when pressing Ctrl+A.
     */
    export function selectAll(target: Element): void;
    export function isAllSelected(target: Element): boolean;
}
declare module "keyboard/types" {
    import { Config } from "setup/index";
    /**
     * @internal Do not create/alter this by yourself as this type might be subject to changes.
     */
    export type keyboardState = {
        /**
            All keys that have been pressed and not been lifted up yet.
          */
        pressed: {
            keyDef: keyboardKey;
            unpreventedDefault: boolean;
        }[];
        /**
            Active modifiers
          */
        modifiers: {
            Alt: boolean;
            AltGraph: boolean;
            CapsLock: boolean;
            Control: boolean;
            Fn: boolean;
            FnLock: boolean;
            Meta: boolean;
            NumLock: boolean;
            ScrollLock: boolean;
            Shift: boolean;
            Symbol: boolean;
            SymbolLock: boolean;
        };
        modifierPhase: {
            CapsLock?: boolean;
            FnLock?: boolean;
            NumLock?: boolean;
            ScrollLock?: boolean;
            SymbolLock?: boolean;
        };
        /**
            The element the keyboard input is performed on.
            Some behavior might differ if the activeElement changes between corresponding keyboard events.
          */
        activeElement: Element | null;
        /**
            For HTMLInputElements type='number':
            If the last input char is '.', '-' or 'e',
            the IDL value attribute does not reflect the input value.
      
            @deprecated The document state workaround in `src/document/value.ts` keeps track
            of UI value diverging from value property.
          */
        carryValue?: string;
        /**
            Carry over characters to following key handlers.
            E.g. ^1
          */
        carryChar: string;
    };
    export enum DOM_KEY_LOCATION {
        STANDARD = 0,
        LEFT = 1,
        RIGHT = 2,
        NUMPAD = 3
    }
    export interface keyboardKey {
        /** Physical location on a keyboard */
        code?: string;
        /** Character or functional key descriptor */
        key?: string;
        /** Location on the keyboard for keys with multiple representation */
        location?: DOM_KEY_LOCATION;
        /** Does the character in `key` require/imply AltRight to be pressed? */
        altGr?: boolean;
        /** Does the character in `key` require/imply a shiftKey to be pressed? */
        shift?: boolean;
    }
    export interface behaviorPlugin {
        matches: (keyDef: keyboardKey, element: Element, config: Config) => boolean;
        handle: (keyDef: keyboardKey, element: Element, config: Config) => void;
    }
}
declare module "keyboard/plugins/arrow" {
    /**
     * This file should contain behavior for arrow keys as described here:
     * https://w3c.github.io/uievents-code/#key-arrowpad-section
     */
    import { behaviorPlugin } from "keyboard/types";
    export const keydownBehavior: behaviorPlugin[];
}
declare module "keyboard/plugins/control" {
    /**
     * This file should contain behavior for arrow keys as described here:
     * https://w3c.github.io/uievents-code/#key-controlpad-section
     */
    import { behaviorPlugin } from "keyboard/types";
    export const keydownBehavior: behaviorPlugin[];
}
declare module "keyboard/shared/fireChangeForInputTimeIfValid" {
    export function fireChangeForInputTimeIfValid(el: HTMLInputElement & {
        type: 'time';
    }, prevValue: unknown, timeNewEntry: string): void;
}
declare module "keyboard/shared/index" {
    export * from "keyboard/shared/fireChangeForInputTimeIfValid";
}
declare module "keyboard/plugins/character" {
    import { behaviorPlugin } from "keyboard/types";
    export const keypressBehavior: behaviorPlugin[];
}
declare module "keyboard/plugins/functional" {
    import { behaviorPlugin } from "keyboard/types";
    export const keydownBehavior: behaviorPlugin[];
    export const keypressBehavior: behaviorPlugin[];
    export const keyupBehavior: behaviorPlugin[];
}
declare module "keyboard/plugins/combination" {
    /**
     * Default behavior for key combinations
     */
    import { behaviorPlugin } from "keyboard/types";
    export const keydownBehavior: behaviorPlugin[];
}
declare module "keyboard/plugins/modifiers" {
    import { behaviorPlugin } from "keyboard/types";
    export const preKeydownBehavior: behaviorPlugin[];
    export const preKeyupBehavior: behaviorPlugin[];
    export const postKeyupBehavior: behaviorPlugin[];
}
declare module "keyboard/plugins/index" {
    import { behaviorPlugin } from "keyboard/types";
    export const preKeydownBehavior: behaviorPlugin[];
    export const keydownBehavior: behaviorPlugin[];
    export const keypressBehavior: behaviorPlugin[];
    export const preKeyupBehavior: behaviorPlugin[];
    export const keyupBehavior: behaviorPlugin[];
    export const postKeyupBehavior: behaviorPlugin[];
}
declare module "keyboard/keyboardAction" {
    import { Config } from "setup/index";
    import { keyboardKey } from "keyboard/types";
    export interface KeyboardAction {
        keyDef: keyboardKey;
        releasePrevious: boolean;
        releaseSelf: boolean;
        repeat: number;
    }
    export function keyboardAction(config: Config, actions: KeyboardAction[]): Promise<void>;
    export function releaseAllKeys(config: Config): Promise<void>;
}
declare module "keyboard/parseKeyDef" {
    import { keyboardKey } from "keyboard/types";
    /**
     * Parse key defintions per `keyboardMap`
     *
     * Keys can be referenced by `{key}` or `{special}` as well as physical locations per `[code]`.
     * Everything else will be interpreted as a typed character - e.g. `a`.
     * Brackets `{` and `[` can be escaped by doubling - e.g. `foo[[bar` translates to `foo[bar`.
     * Keeping the key pressed can be written as `{key>}`.
     * When keeping the key pressed you can choose how long (how many keydown and keypress) the key is pressed `{key>3}`.
     * You can then release the key per `{key>3/}` or keep it pressed and continue with the next key.
     */
    export function parseKeyDef(keyboardMap: keyboardKey[], text: string): {
        keyDef: keyboardKey;
        releasePrevious: boolean;
        releaseSelf: boolean;
        repeat: number;
    }[];
}
declare module "keyboard/index" {
    import { UserEvent } from "setup/index";
    import { releaseAllKeys } from "keyboard/keyboardAction";
    import type { keyboardState, keyboardKey } from "keyboard/types";
    export { releaseAllKeys };
    export type { keyboardKey, keyboardState };
    export function keyboard(this: UserEvent, text: string): Promise<void>;
    export function createKeyboardState(): keyboardState;
}
declare module "utils/keyboard/getUIEventModifiers" {
    import type { keyboardState } from "keyboard/index";
    export function getUIEventModifiers(keyboardState: keyboardState): {
        altKey: boolean;
        ctrlKey: boolean;
        metaKey: boolean;
        shiftKey: boolean;
        modifierAltGraph: boolean;
        modifierCapsLock: boolean;
        modifierFn: boolean;
        modifierFnLock: boolean;
        modifierNumLock: boolean;
        modifierScrollLock: boolean;
        modifierSymbol: boolean;
        modifierSymbolLock: boolean;
    };
}
declare module "utils/keyboard/getKeyEventProps" {
    import { keyboardKey, keyboardState } from "keyboard/types";
    export function getKeyEventProps(keyDef: keyboardKey, state: keyboardState): {
        altKey: boolean;
        ctrlKey: boolean;
        metaKey: boolean;
        shiftKey: boolean;
        modifierAltGraph: boolean;
        modifierCapsLock: boolean;
        modifierFn: boolean;
        modifierFnLock: boolean;
        modifierNumLock: boolean;
        modifierScrollLock: boolean;
        modifierSymbol: boolean;
        modifierSymbolLock: boolean;
        key: string | undefined;
        code: string | undefined;
    };
}
declare module "utils/keyDef/readNextDescriptor" {
    /**
     * Read the next key definition from user input
     *
     * Describe key per `{descriptor}` or `[descriptor]`.
     * Everything else will be interpreted as a single character as descriptor - e.g. `a`.
     * Brackets `{` and `[` can be escaped by doubling - e.g. `foo[[bar` translates to `foo[bar`.
     * A previously pressed key can be released per `{/descriptor}`.
     * Keeping the key pressed can be written as `{descriptor>}`.
     * When keeping the key pressed you can choose how long the key is pressed `{descriptor>3}`.
     * You can then release the key per `{descriptor>3/}` or keep it pressed and continue with the next key.
     */
    export function readNextDescriptor(text: string): {
        consumedLength: number;
        descriptor: string;
        releasePrevious: boolean;
        repeat: number;
        releaseSelf: boolean | undefined;
        type: string;
    };
}
declare module "utils/misc/findClosest" {
    export function findClosest(element: Element, callback: (e: Element) => boolean): Element | undefined;
}
declare module "utils/misc/getDocumentFromNode" {
    export function getDocumentFromNode(el: Node): Document | null;
}
declare module "utils/misc/isDescendantOrSelf" {
    export function isDescendantOrSelf(potentialDescendant: Element, potentialAncestor: Element): boolean;
}
declare module "utils/misc/wait" {
    export function wait(time?: number): Promise<void>;
}
declare module "utils/misc/hasPointerEvents" {
    /**
     * Options that can be passed to any event that relies
     * on pointer-events property
     */
    export interface PointerOptions {
        /**
         * When set to `true` the event skips checking if any element
         * in the DOM-tree has `'pointer-events: none'` set. This check is
         * costly in general  and very costly when rendering large DOM-trees.
         * Can be used to speed up tests.
         * Default: `false`
         * */
        skipPointerEventsCheck?: boolean;
    }
    export function hasPointerEvents(element: Element): boolean;
}
declare module "utils/misc/hasFormSubmit" {
    export const hasFormSubmit: (form: HTMLFormElement | null) => form is HTMLFormElement;
}
declare module "pointer/types" {
    import { PointerCoords, MouseButton } from "utils/index";
    /**
     * @internal Do not create/alter this by yourself as this type might be subject to changes.
     */
    export type pointerState = {
        /**
              All keys that have been pressed and not been lifted up yet.
            */
        pressed: {
            keyDef: pointerKey;
            pointerId: number;
            isMultiTouch: boolean;
            isPrimary: boolean;
            clickCount: number;
            unpreventedDefault: boolean;
            /** Target the key was pressed on */
            downTarget: Element;
        }[];
        activeClickCount?: [string, number];
        /**
         * Position of each pointer.
         * The mouse is always pointer 1 and keeps its position.
         * Pen and touch devices receive a new pointerId for every interaction.
         */
        position: Record<string, {
            pointerId: number;
            pointerType: 'mouse' | 'pen' | 'touch';
        } & Partial<PointerTarget> & {
            selectionRange?: Range | SelectionInputRange;
        }>;
        /**
         * Last applied pointer id
         */
        pointerId: number;
    };
    export interface pointerKey {
        /** Name of the pointer key */
        name: string;
        /** Type of the pointer device */
        pointerType: 'mouse' | 'pen' | 'touch';
        /** Type of button */
        button?: MouseButton;
    }
    export interface PointerTarget {
        target: Element;
        coords?: PointerCoords;
    }
    export interface SelectionTarget {
        node?: Node;
        /**
         * If `node` is set, this is the DOM offset.
         * Otherwise this is the `textContent`/`value` offset on the `target`.
         */
        offset?: number;
    }
    /** Describes a selection inside `HTMLInputElement`/`HTMLTextareaElement` */
    export interface SelectionInputRange {
        node: HTMLInputElement | HTMLTextAreaElement;
        start: number;
        end: number;
    }
}
declare module "utils/pointer/mouseButtons" {
    export const MouseButton: {
        readonly primary: 0;
        readonly secondary: 1;
        readonly auxiliary: 2;
        readonly back: 3;
        readonly X1: 3;
        readonly forward: 4;
        readonly X2: 4;
    };
    export type MouseButton = keyof typeof MouseButton | number;
    export function getMouseButton(button: MouseButton): number;
    export function getMouseButtons(...buttons: Array<MouseButton>): number;
}
declare module "utils/pointer/firePointerEvents" {
    import type { pointerState } from "pointer/types";
    import type { keyboardState } from "keyboard/types";
    import { MouseButton } from "utils/pointer/mouseButtons";
    export function firePointerEvent(target: Element, type: string, { pointerState, keyboardState, pointerType, button, coords, pointerId, isPrimary, clickCount, }: {
        pointerState: pointerState;
        keyboardState: keyboardState;
        pointerType?: 'mouse' | 'pen' | 'touch';
        button?: MouseButton;
        coords?: PointerCoords;
        pointerId?: number;
        isPrimary?: boolean;
        clickCount?: number;
    }): boolean;
    export interface PointerCoords {
        x?: number;
        y?: number;
        clientX?: number;
        clientY?: number;
        offsetX?: number;
        offsetY?: number;
        pageX?: number;
        pageY?: number;
        screenX?: number;
        screenY?: number;
    }
}
declare module "utils/index" {
    export * from "utils/click/isClickableInput";
    export * from "utils/dataTransfer/Blob";
    export * from "utils/dataTransfer/DataTransfer";
    export * from "utils/dataTransfer/FileList";
    export * from "utils/dataTransfer/Clipboard";
    export * from "utils/edit/buildTimeValue";
    export * from "utils/edit/calculateNewValue";
    export * from "utils/edit/editInputElement";
    export * from "utils/edit/getValue";
    export * from "utils/edit/isContentEditable";
    export * from "utils/edit/isEditable";
    export * from "utils/edit/isValidDateValue";
    export * from "utils/edit/isValidInputTimeValue";
    export * from "utils/edit/maxLength";
    export * from "utils/edit/prepareInput";
    export * from "utils/edit/setFiles";
    export * from "utils/focus/blur";
    export * from "utils/focus/copySelection";
    export * from "utils/focus/focus";
    export * from "utils/focus/getActiveElement";
    export * from "utils/focus/getTabDestination";
    export * from "utils/focus/isFocusable";
    export * from "utils/focus/selectAll";
    export * from "utils/focus/selection";
    export * from "utils/focus/selector";
    export * from "utils/keyboard/getKeyEventProps";
    export * from "utils/keyboard/getUIEventModifiers";
    export * from "utils/keyDef/readNextDescriptor";
    export * from "utils/misc/eventWrapper";
    export * from "utils/misc/findClosest";
    export * from "utils/misc/getDocumentFromNode";
    export * from "utils/misc/isDescendantOrSelf";
    export * from "utils/misc/isElementType";
    export * from "utils/misc/isVisible";
    export * from "utils/misc/isDisabled";
    export * from "utils/misc/wait";
    export * from "utils/misc/hasPointerEvents";
    export * from "utils/misc/hasFormSubmit";
    export * from "utils/pointer/firePointerEvents";
    export * from "utils/pointer/mouseButtons";
}
declare module "pointer/parseKeyDef" {
    import { pointerKey } from "pointer/types";
    export function parseKeyDef(pointerMap: pointerKey[], keys: string): {
        keyDef: pointerKey;
        releasePrevious: boolean;
        releaseSelf: boolean;
    }[];
}
declare module "pointer/resolveSelectionTarget" {
    import { PointerTarget, SelectionTarget } from "pointer/types";
    export function resolveSelectionTarget({ target, node, offset, }: PointerTarget & SelectionTarget): {
        node: Node;
        offset: number;
    };
}
declare module "pointer/pointerMove" {
    import { inputDeviceState } from "setup/index";
    import { PointerTarget, SelectionTarget } from "pointer/types";
    export interface PointerMoveAction extends PointerTarget, SelectionTarget {
        pointerName?: string;
    }
    export function pointerMove({ pointerName, target, coords, node, offset }: PointerMoveAction, { pointerState, keyboardState }: inputDeviceState): Promise<void>;
}
declare module "pointer/pointerPress" {
    import { inputDeviceState } from "setup/index";
    import type { pointerKey, PointerTarget, SelectionTarget } from "pointer/types";
    export interface PointerPressAction extends PointerTarget, SelectionTarget {
        keyDef: pointerKey;
        releasePrevious: boolean;
        releaseSelf: boolean;
    }
    export function pointerPress(action: PointerPressAction, state: inputDeviceState): Promise<void>;
}
declare module "pointer/pointerAction" {
    import { Config } from "setup/index";
    import { PointerMoveAction } from "pointer/pointerMove";
    import { PointerPressAction } from "pointer/pointerPress";
    import { PointerTarget, SelectionTarget } from "pointer/types";
    export type PointerActionTarget = Partial<PointerTarget> & Partial<SelectionTarget>;
    export type PointerAction = PointerActionTarget & (Omit<PointerPressAction, 'target' | 'coords'> | Omit<PointerMoveAction, 'target' | 'coords'>);
    export function pointerAction(config: Config, actions: PointerAction[]): Promise<void>;
}
declare module "pointer/index" {
    import { UserEvent } from "setup/index";
    import { PointerAction, PointerActionTarget } from "pointer/pointerAction";
    import type { pointerState, pointerKey } from "pointer/types";
    export type { pointerState, pointerKey };
    type PointerActionInput = string | ({
        keys: string;
    } & PointerActionTarget) | PointerAction;
    export type PointerInput = PointerActionInput | Array<PointerActionInput>;
    export function pointer(this: UserEvent, input: PointerInput): Promise<void>;
    export function createPointerState(document: Document): pointerState;
}
declare module "convenience/click" {
    import { UserEvent } from "setup/index";
    export function click(this: UserEvent, element: Element): Promise<void>;
    export function dblClick(this: UserEvent, element: Element): Promise<void>;
    export function tripleClick(this: UserEvent, element: Element): Promise<void>;
}
declare module "convenience/hover" {
    import { UserEvent } from "setup/index";
    export function hover(this: UserEvent, element: Element): Promise<void>;
    export function unhover(this: UserEvent, element: Element): Promise<void>;
}
declare module "convenience/tab" {
    import type { UserEvent } from "setup/index";
    export function tab(this: UserEvent, { shift, }?: {
        shift?: boolean;
    }): Promise<void>;
}
declare module "convenience/index" {
    export * from "convenience/click";
    export * from "convenience/hover";
    export * from "convenience/tab";
}
declare module "clipboard/copy" {
    import { UserEvent } from "setup/index";
    export function copy(this: UserEvent): Promise<DataTransfer | undefined>;
}
declare module "clipboard/cut" {
    import { UserEvent } from "setup/index";
    export function cut(this: UserEvent): Promise<DataTransfer | undefined>;
}
declare module "clipboard/paste" {
    import { UserEvent } from "setup/index";
    export function paste(this: UserEvent, clipboardData?: DataTransfer | string): Promise<void>;
}
declare module "clipboard/index" {
    export * from "clipboard/copy";
    export * from "clipboard/cut";
    export * from "clipboard/paste";
}
declare module "utility/clear" {
    import type { UserEvent } from "setup/index";
    export function clear(this: UserEvent, element: Element): Promise<void>;
}
declare module "utility/selectOptions" {
    import { UserEvent } from "setup/index";
    export function selectOptions(this: UserEvent, select: Element, values: HTMLElement | HTMLElement[] | string[] | string): Promise<void>;
    export function deselectOptions(this: UserEvent, select: Element, values: HTMLElement | HTMLElement[] | string[] | string): Promise<void>;
}
declare module "keyboard/keyMap" {
    import { keyboardKey } from "keyboard/types";
    /**
     * Mapping for a default US-104-QWERTY keyboard
     */
    export const defaultKeyMap: keyboardKey[];
}
declare module "pointer/keyMap" {
    import { pointerKey } from "pointer/types";
    export const defaultKeyMap: pointerKey[];
}
declare module "options" {
    import type { keyboardKey } from "keyboard/types";
    import type { pointerKey } from "pointer/types";
    export interface Options {
        /**
         * When using `userEvent.upload`, automatically discard files
         * that don't match an `accept` property if it exists.
         *
         * @default true
         */
        applyAccept?: boolean;
        /**
         * We intend to automatically apply modifier keys for printable characters in the future.
         * I.e. `A` implying `{Shift>}a{/Shift}` if caps lock is not active.
         *
         * This options allows you to opt out of this change in foresight.
         * The feature therefore will not constitute a breaking change.
         *
         * @default true
         */
        autoModify?: boolean;
        /**
         * Between some subsequent inputs like typing a series of characters
         * the code execution is delayed per `setTimeout` for (at least) `delay` seconds.
         * This moves the next changes at least to next macro task
         * and allows other (asynchronous) code to run between events.
         *
         * `null` prevents `setTimeout` from being called.
         *
         * @default 0
         */
        delay?: number | null;
        /**
         * The document.
         *
         * This defaults to the owner document of an element if an API is called directly with an element and without setup.
         * Otherwise it falls back to the global document.
         *
         * @default element.ownerDocument??global.document
         */
        document?: Document;
        /**
         * An array of keyboard keys the keyboard device consists of.
         *
         * This allows to plug in different layouts / localizations.
         *
         * Defaults to a "standard" US-104-QWERTY keyboard.
         */
        keyboardMap?: keyboardKey[];
        /**
         * An array of available pointer keys.
         *
         * This allows to plug in different pointer devices.
         */
        pointerMap?: pointerKey[];
        /**
         * `userEvent.type` automatically releases any keys still pressed at the end of the call.
         * This option allows to opt out of this feature.
         *
         * @default false
         */
        skipAutoClose?: boolean;
        /**
         * `userEvent.type` implys a click at the end of the element content/value.
         * This option allows to opt out of this feature.
         *
         * @default false
         */
        skipClick?: boolean;
        /**
         * `userEvent.click` implys moving the cursor to the target element first.
         * This options allows to opt out of this feature.
         *
         * @default false
         */
        skipHover?: boolean;
        /**
         * Calling pointer related APIs on an element triggers a check if that element can receive pointer events.
         * This check is known to be expensive.
         * This option allows to skip the check.
         *
         * @default false
         */
        skipPointerEventsCheck?: boolean;
        /**
         * Write selected data to Clipboard API when a `cut` or `copy` is triggered.
         *
         * The Clipboard API is usually not available to test code.
         * Our `setup` replaces the `navigator.clipboard` property with a stub.
         *
         * Defaults to `false` when calling the APIs directly.
         * Defaults to `true` when calling the APIs per `setup`.
         */
        writeToClipboard?: boolean;
    }
    /**
     * Default options applied when API is called per `userEvent.anyApi()`
     */
    export const defaultOptionsDirect: Required<Options>;
    /**
     * Default options applied when API is called per `userEvent().anyApi()`
     */
    export const defaultOptionsSetup: Required<Options>;
}
declare module "setup/config" {
    import type { keyboardState } from "keyboard/types";
    import type { pointerState } from "pointer/types";
    import type { Options } from "options";
    export interface inputDeviceState {
        pointerState: pointerState;
        keyboardState: keyboardState;
    }
    export interface Config extends Required<Options>, inputDeviceState {
    }
    export const Config: unique symbol;
}
declare module "utility/type" {
    import type { UserEvent } from "setup/index";
    import { Config } from "setup/config";
    export interface typeOptions {
        skipClick?: Config['skipClick'];
        skipAutoClose?: Config['skipClick'];
        initialSelectionStart?: number;
        initialSelectionEnd?: number;
    }
    export function type(this: UserEvent, element: Element, text: string, { skipClick, skipAutoClose, initialSelectionStart, initialSelectionEnd, }?: typeOptions): Promise<void>;
}
declare module "utility/upload" {
    import { UserEvent } from "setup/index";
    export interface uploadInit {
        changeInit?: EventInit;
    }
    export function upload(this: UserEvent, element: HTMLElement, fileOrFiles: File | File[]): Promise<void>;
}
declare module "utility/index" {
    export * from "utility/clear";
    export * from "utility/selectOptions";
    export * from "utility/type";
    export * from "utility/upload";
}
declare module "setup/api" {
    export { click, dblClick, tripleClick, hover, unhover, tab } from "convenience/index";
    export { keyboard } from "keyboard/index";
    export { copy, cut, paste } from "clipboard/index";
    export { pointer } from "pointer/index";
    export { clear, deselectOptions, selectOptions, type, upload } from "utility/index";
}
declare module "setup/wrapAsync" {
    /**
     * Wrap an internal Promise
     */
    export function wrapAsync<R, P extends (() => Promise<R>) | (() => R)>(implementation: P): Promise<R>;
}
declare module "setup/setup" {
    import { Options } from "options";
    import type { UserEvent } from "setup/index";
    import { Config } from "setup/config";
    import * as userEventApi from "setup/api";
    /**
     * Start a "session" with userEvent.
     * All APIs returned by this function share an input device state and a default configuration.
     */
    export function setupMain(options?: Options): {
        setup: typeof setupSub;
        [Config]: Config;
        click: typeof userEventApi.click;
        dblClick: typeof userEventApi.dblClick;
        tripleClick: typeof userEventApi.tripleClick;
        hover: typeof userEventApi.hover;
        unhover: typeof userEventApi.unhover;
        tab: typeof userEventApi.tab;
        keyboard: typeof userEventApi.keyboard;
        copy: typeof userEventApi.copy;
        cut: typeof userEventApi.cut;
        paste: typeof userEventApi.paste;
        pointer: typeof userEventApi.pointer;
        clear: typeof userEventApi.clear;
        deselectOptions: typeof userEventApi.deselectOptions;
        selectOptions: typeof userEventApi.selectOptions;
        type: typeof userEventApi.type;
        upload: typeof userEventApi.upload;
    };
    /**
     * Setup in direct call per `userEvent.anyApi()`
     */
    export function setupDirect(options?: Partial<Config>, node?: Node): {
        setup: typeof setupSub;
        [Config]: Config;
        click: typeof userEventApi.click;
        dblClick: typeof userEventApi.dblClick;
        tripleClick: typeof userEventApi.tripleClick;
        hover: typeof userEventApi.hover;
        unhover: typeof userEventApi.unhover;
        tab: typeof userEventApi.tab;
        keyboard: typeof userEventApi.keyboard;
        copy: typeof userEventApi.copy;
        cut: typeof userEventApi.cut;
        paste: typeof userEventApi.paste;
        pointer: typeof userEventApi.pointer;
        clear: typeof userEventApi.clear;
        deselectOptions: typeof userEventApi.deselectOptions;
        selectOptions: typeof userEventApi.selectOptions;
        type: typeof userEventApi.type;
        upload: typeof userEventApi.upload;
    };
    /**
     * Create a set of callbacks with different default settings but the same state.
     */
    export function setupSub(this: UserEvent, options: Options): {
        setup: typeof setupSub;
        [Config]: Config;
        click: typeof userEventApi.click;
        dblClick: typeof userEventApi.dblClick;
        tripleClick: typeof userEventApi.tripleClick;
        hover: typeof userEventApi.hover;
        unhover: typeof userEventApi.unhover;
        tab: typeof userEventApi.tab;
        keyboard: typeof userEventApi.keyboard;
        copy: typeof userEventApi.copy;
        cut: typeof userEventApi.cut;
        paste: typeof userEventApi.paste;
        pointer: typeof userEventApi.pointer;
        clear: typeof userEventApi.clear;
        deselectOptions: typeof userEventApi.deselectOptions;
        selectOptions: typeof userEventApi.selectOptions;
        type: typeof userEventApi.type;
        upload: typeof userEventApi.upload;
    };
}
declare module "setup/directApi" {
    import type { PointerOptions } from "utils/index";
    import type { PointerInput } from "pointer/index";
    import type { UserEventApi } from "setup/index";
    import { Config } from "setup/config";
    export function clear(element: Element): Promise<void>;
    export function click(element: Element, options?: Partial<Config>): Promise<void>;
    export function copy(options?: Partial<Config>): Promise<DataTransfer | undefined>;
    export function cut(options?: Partial<Config>): Promise<DataTransfer | undefined>;
    export function dblClick(element: Element, options?: Partial<Config>): Promise<void>;
    export function deselectOptions(select: Element, values: HTMLElement | HTMLElement[] | string[] | string, options?: Partial<Config>): Promise<void>;
    export function hover(element: Element, options?: Partial<Config>): Promise<void>;
    export function keyboard(text: string, options?: Partial<Config>): Promise<import("keyboard").keyboardState>;
    export function pointer(input: PointerInput, options?: Partial<Config>): Promise<import("pointer").pointerState>;
    export function paste(clipboardData?: DataTransfer | string, options?: Partial<Config>): Promise<void>;
    export function selectOptions(select: Element, values: HTMLElement | HTMLElement[] | string[] | string, options?: Partial<Config>): Promise<void>;
    export function tripleClick(element: Element, options?: Partial<Config>): Promise<void>;
    export function type(element: Element, text: string, options?: Partial<Config> & Parameters<UserEventApi['type']>[2]): Promise<void>;
    export function unhover(element: Element, options?: PointerOptions): Promise<void>;
    export function upload(element: HTMLElement, fileOrFiles: File | File[], options?: Partial<Config>): Promise<void>;
    export function tab(options?: Partial<Config> & Parameters<UserEventApi['tab']>[0]): Promise<void>;
}
declare module "setup/index" {
    import type * as userEventApi from "setup/api";
    import { setupMain, setupSub } from "setup/setup";
    import { Config, inputDeviceState } from "setup/config";
    import * as directApi from "setup/directApi";
    export type { inputDeviceState };
    export { Config };
    export type UserEventApi = typeof userEventApi;
    export type UserEvent = UserEventApi & {
        readonly setup: typeof setupSub;
        [Config]: Config;
    };
    export const userEvent: {
        readonly setup: typeof setupMain;
        readonly clear: typeof directApi.clear;
        readonly click: typeof directApi.click;
        readonly copy: typeof directApi.copy;
        readonly cut: typeof directApi.cut;
        readonly dblClick: typeof directApi.dblClick;
        readonly deselectOptions: typeof directApi.deselectOptions;
        readonly hover: typeof directApi.hover;
        readonly keyboard: typeof directApi.keyboard;
        readonly pointer: typeof directApi.pointer;
        readonly paste: typeof directApi.paste;
        readonly selectOptions: typeof directApi.selectOptions;
        readonly tripleClick: typeof directApi.tripleClick;
        readonly type: typeof directApi.type;
        readonly unhover: typeof directApi.unhover;
        readonly upload: typeof directApi.upload;
        readonly tab: typeof directApi.tab;
    };
}
declare module "index" {
    export { userEvent as default } from "setup/index";
    export type { keyboardKey } from "keyboard/index";
    export type { pointerKey } from "pointer/index";
}
